"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SQLServer = void 0;
const native_1 = require("@cubejs-backend/native");
const shared_1 = require("@cubejs-backend/shared");
const crypto = __importStar(require("crypto"));
class SQLServer {
    constructor(apiGateway) {
        this.apiGateway = apiGateway;
        this.sqlInterfaceInstance = null;
        (0, native_1.setupLogger)(({ event }) => apiGateway.log(event), process.env.CUBEJS_LOG_LEVEL === 'trace' ? 'trace' : 'warn');
    }
    async init(options) {
        if (this.sqlInterfaceInstance) {
            throw new Error('Unable to start SQL interface two times');
        }
        const checkSqlAuth = (options.checkSqlAuth && this.wrapCheckSqlAuthFn(options.checkSqlAuth))
            || this.createDefaultCheckSqlAuthFn(options);
        const canSwitchSqlUser = options.canSwitchSqlUser
            || this.createDefaultCanSwitchSqlUserFn(options);
        const contextByRequest = async (request, session) => {
            let userForContext = session.user;
            if (request.meta.changeUser && request.meta.changeUser !== session.user) {
                const canSwitch = session.superuser || await canSwitchSqlUser(session.user, request.meta.changeUser);
                if (canSwitch) {
                    userForContext = request.meta.changeUser;
                }
                else {
                    throw new Error(`You cannot change security context via __user from ${session.user} to ${request.meta.changeUser}, because it's not allowed.`);
                }
            }
            // @todo Store security context in native for session's user, but not for switching
            const current = await checkSqlAuth(request, userForContext);
            return this.contextByNativeReq(request, current.securityContext, request.id);
        };
        const canSwitchUserForSession = async (session, user) => session.superuser || canSwitchSqlUser(session.user, user);
        this.sqlInterfaceInstance = await (0, native_1.registerInterface)({
            port: options.sqlPort,
            pgPort: options.pgSqlPort,
            nonce: options.sqlNonce,
            checkAuth: async ({ request, user }) => {
                const { password, superuser } = await checkSqlAuth(request, user);
                // Strip securityContext to improve speed deserialization
                return {
                    password,
                    superuser: superuser || false,
                };
            },
            meta: async ({ request, session }) => {
                // @todo Store security context in native
                const { securityContext } = await checkSqlAuth(request, session.user);
                const context = await this.apiGateway.contextByReq(request, securityContext, request.id);
                // eslint-disable-next-line no-async-promise-executor
                return new Promise(async (resolve, reject) => {
                    try {
                        await this.apiGateway.meta({
                            context,
                            res: (message) => {
                                resolve(message);
                            },
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            },
            load: async ({ request, session, query }) => {
                const context = await contextByRequest(request, session);
                // eslint-disable-next-line no-async-promise-executor
                return new Promise(async (resolve, reject) => {
                    try {
                        await this.apiGateway.load({
                            query,
                            queryType: 'multi',
                            context,
                            res: (message) => {
                                resolve(message);
                            },
                            apiType: 'sql',
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            },
            sqlApiLoad: async ({ request, session, query, sqlQuery, streaming }) => {
                const context = await contextByRequest(request, session);
                // eslint-disable-next-line no-async-promise-executor
                return new Promise(async (resolve, reject) => {
                    try {
                        await this.apiGateway.sqlApiLoad({
                            query,
                            sqlQuery,
                            streaming,
                            context,
                            res: (message) => {
                                resolve(message);
                            },
                            apiType: 'sql',
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            },
            sql: async ({ request, session, query, memberToAlias, expressionParams }) => {
                const context = await contextByRequest(request, session);
                // eslint-disable-next-line no-async-promise-executor
                return new Promise(async (resolve, reject) => {
                    try {
                        await this.apiGateway.sql({
                            query,
                            memberToAlias,
                            expressionParams,
                            exportAnnotatedSql: true,
                            memberExpressions: true,
                            queryType: 'multi',
                            context,
                            res: (message) => {
                                resolve(message);
                            },
                            apiType: 'sql',
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            },
            stream: async ({ request, session, query }) => {
                const context = await contextByRequest(request, session);
                // eslint-disable-next-line no-async-promise-executor
                return new Promise(async (resolve, reject) => {
                    try {
                        resolve(await this.apiGateway.stream(context, query));
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            },
            sqlGenerators: async (paramsJson) => {
                // TODO get rid of it
                const { request, session } = JSON.parse(paramsJson);
                // @todo Store security context in native
                const { securityContext } = await checkSqlAuth(request, session.user);
                const context = await this.apiGateway.contextByReq(request, securityContext, request.id);
                // eslint-disable-next-line no-async-promise-executor
                return new Promise(async (resolve, reject) => {
                    try {
                        await this.apiGateway.sqlGenerators({
                            context,
                            res: (queries) => {
                                resolve(queries);
                            },
                        });
                    }
                    catch (e) {
                        reject(e);
                    }
                });
            },
            canSwitchUserForSession: async ({ session, user }) => canSwitchUserForSession(session, user)
        });
    }
    wrapCheckSqlAuthFn(checkSqlAuth) {
        return async (req, user) => {
            const response = await checkSqlAuth(req, user);
            if (typeof response !== 'object' || response.password === null) {
                throw new Error('checkSqlAuth must return an object');
            }
            if (!response.password) {
                throw new Error('checkSqlAuth must return an object with password field');
            }
            return response;
        };
    }
    createDefaultCanSwitchSqlUserFn(options) {
        const superUser = options.sqlSuperUser || (0, shared_1.getEnv)('sqlSuperUser');
        return async (current, _user) => {
            if (superUser) {
                return current === superUser;
            }
            return false;
        };
    }
    createDefaultCheckSqlAuthFn(options) {
        let allowedUser = options.sqlUser || (0, shared_1.getEnv)('sqlUser');
        let allowedPassword = options.sqlPassword || (0, shared_1.getEnv)('sqlPassword');
        if (!(0, shared_1.getEnv)('devMode')) {
            if (!allowedUser) {
                allowedUser = 'cube';
                (0, shared_1.displayCLIWarning)('Option sqlUser is required in production mode. Cube.js will use \'cube\' as a default username.');
            }
            if (!allowedPassword) {
                allowedPassword = crypto.randomBytes(16).toString('hex');
                (0, shared_1.displayCLIWarning)(`Option sqlPassword is required in production mode. Cube.js has generated it as '${allowedPassword}'`);
            }
        }
        return async (req, user) => {
            if (allowedUser && user !== allowedUser) {
                throw new Error(`Incorrect user name "${user}" or password`);
            }
            return {
                password: allowedPassword,
                securityContext: {}
            };
        };
    }
    async contextByNativeReq(req, securityContext, requestId) {
        const context = await this.apiGateway.contextByReq(req, securityContext, requestId);
        return {
            ...context,
            ...req.meta,
        };
    }
    async close() {
        // @todo Implement
    }
}
exports.SQLServer = SQLServer;
//# sourceMappingURL=sql-server.js.map